@@ -2,9 +2,9 @@
 # This code is part of the MAterials Simulation Toolkit (MAST)
 # 
 # Maintainer: Tam Mayeshiba
-# Last updated: 2014-05-12 by Zhewen Song
+# Last updated: 2014-06-13 by Zhewen Song
 ##############################################################
-import os
+import os, re
 import time
 import fnmatch
 import logging
@@ -53,6 +53,9 @@ INGREDIENTS_KEYWORDS = ['singlepoint',
 RECIPE_KEYWORDS = {'recipe_file': None,
                   }
 
+PERS_RECP_KEYWORDS = {'personal_recipe_list': None,
+                      }
+
 class InputParser(MASTObj):
     """Class for parsing a *.inp format input file.
         Attributes:
@@ -73,9 +76,10 @@ class InputParser(MASTObj):
                 'recipe'   : self.parse_recipe_section,
                 'neb'      : self.parse_neb_section,
                 'chemical_potentials' : self.parse_chemical_potentials_section,
-                'summary' : self.parse_summary_section
+                'summary' : self.parse_summary_section,
+                'personal_recipe' : self.parse_personal_recipe_section
                                }
-        scratchpath = os.getenv("MAST_SCRATCH").strip('/')
+        scratchpath = dirutil.get_mast_scratch_path()
         inputlocation = os.path.dirname(self.keywords['inputfile'])
         if (inputlocation == ""):
             self.logger = logging.getLogger('mast')
@@ -128,7 +132,8 @@ class InputParser(MASTObj):
                         section_content, options)
                 self.logger.info('Finished parsing the %s section.' % section_name)
             else:
-                line = line.strip()
+                if (section_name != 'recipe') and (section_name != 'personal_recipe'): 
+                        line = line.strip()
                 if (line):
                     section_content.append(line)
         infile.close()
@@ -144,7 +149,7 @@ class InputParser(MASTObj):
         mast_dict = MAST_KEYWORDS.copy()
 
         for line in section_content:
-            line = line.split(self.delimiter)
+            line = line.split()
             if (line[0] not in mast_dict):
                 error = 'Section keyword %s not recognized' % line[0]
                 MASTError(self.__class__.__name__, error)
@@ -186,8 +191,13 @@ class InputParser(MASTObj):
         structure_dict = STRUCTURE_KEYWORDS.copy() 
 
         subsection_dict = dict()
+        matrix = []
         for myline in section_content:
-            line = myline.split(self.delimiter, 1)
+            if '[' and ']' in myline:
+                matrix.append(myline.split('[')[1].split(']')[0])
+                myline = myline.split(']')[1].strip()
+                line = myline.split()
+            line = myline.split()
 
             if (line[0] in structure_dict):
                 structure_dict[line[0]] = line[1]
@@ -195,7 +205,7 @@ class InputParser(MASTObj):
                 subsection = line[1]
                 subsection_list = list()
             elif ('end' not in line):
-                lsplit = myline.split(self.delimiter)
+                lsplit = myline.split()
                 lineval = list()
                 for lval in lsplit:
                     lval.strip()
@@ -211,7 +221,9 @@ class InputParser(MASTObj):
         # an error.
         if (structure_dict['posfile'] is not None): # Do we have a geometry file?
             # First build a list of likely files
-            origindir = os.getcwd()
+            origindir = os.path.dirname(self.keywords['inputfile'])
+            if origindir == "":
+                origindir = os.getcwd()
             metatry = os.path.join(os.getcwd(), 'metadata.txt')
             if os.path.isfile(metatry):
                 myrecipemeta = Metadata(metafile=metatry)
@@ -260,14 +272,35 @@ class InputParser(MASTObj):
                     element_map[elkey]=elname
                 structure_dict['element_map'] = element_map
             if (key == 'scaling'):
+                index = 0
+                scsize = []
                 for scline in value:
-                    scsize = scline[0].strip()
-                    sckmesh = scline[1].strip()
-                    scktype = scline[2].strip()
+                    try:
+                        sckmesh = scline[0].strip()
+                    except: raise MASTError(self.__class__.__name__, "No kpoint mesh given!")
                     sckshift = "0.0 0.0 0.0"
-                    try: sckshift = "%s %s %s"%(scline[3],scline[4],scline[5])
+                    scktype = 'M'
+                    try: 
+                        if scline[1].strip().upper()=='M' or scline[1].strip().upper()=='G':
+                            scktype = scline[1].strip()
                     except IndexError: pass
-                    scaling[scsize]=[sckmesh,scktype,sckshift]
+                    if 'label=' in scline[-1]:
+                        scsize.append(scline[-1].strip().split('label=')[1])
+                        try:                                     
+                            sckshift = "%s %s %s"%(float(scline[-4]), float(scline[-3]), float(scline[-2]))
+                        except: pass                            
+                    else:
+                        label = 1
+                        while 'size%s'%label in scsize:
+                            label += 1
+                        scsize.append('size%s'%label)
+                        try: 
+                            sckshift = "%s %s %s"%(float(scline[-3]), float(scline[-2]), float(scline[-1]))
+                        except: pass
+                    if scsize[index] in scaling.keys():
+                        raise MASTError(self.__class__.__name__, "Label %s conflicted with the default label. Please rename this label!"%scsize[index])
+                    scaling[scsize[index]]=[matrix[index],sckmesh,scktype,sckshift]
+                    index += 1
                 structure_dict['scaling'] = scaling
         if len(element_map) > 0 and len(atom_list) > 0:
             new_atom_list = list()
@@ -280,7 +313,6 @@ class InputParser(MASTObj):
 
         for key, value in structure_dict.items():
             options.set_item(section_name, key, value)
-
     def parse_defects_section(self, section_name, section_content, options):
         """Parses the defects section and populates the options.
         """
@@ -293,8 +325,8 @@ class InputParser(MASTObj):
         threshold = 1.e-4
 
         for line in section_content:
-            line = line.split(self.delimiter)
-
+            #line = line.split(self.delimiter)
+            line = line.split()
             if (line[0] == 'coord_type'):
                 coord_type = line[1]
             elif (line[0] == 'threshold'):
@@ -396,24 +428,30 @@ class InputParser(MASTObj):
         """Parses the recipe section and populates the options."""
         recipe_dict = RECIPE_KEYWORDS.copy()
 
-        for line in section_content:
-            line = line.split(self.delimiter)
-            if (line[0] not in recipe_dict):
-                error = 'Section keyword %s not recognized' % line[0]
-                MASTError(self.__class__.__name__, error)
-                return
-            elif (line[0] == 'recipe_file'):
-                try:
-                    recipe_path = os.getenv('MAST_RECIPE_PATH')
-                except KeyError:
-                    error = 'MAST_RECIPE_PATH environment variable not set'
-                    MASTError(self.__class__.__name__, error) 
-                recipe_dict['recipe_file'] = '%s/%s' % (recipe_path, line[1])
-            else:
-                recipe_dict[line[0]] = line[1]
+        if not section_content:
+            error = 'Recipe section is not specified'
+            MASTError(self.__class__.__name__, error)
+            return	
+        else:
+            recipe_dict['recipe_file'] = section_content
+
         for key, value in recipe_dict.items():
             options.set_item(section_name, key, value)
 
+    def parse_personal_recipe_section(self, section_name, section_content, options):
+        """Parses the recipe section and populates the options."""
+        personal_recipe_dict = PERS_RECP_KEYWORDS.copy()
+
+        if not section_content:
+            error = 'Personal Recipe section is not specified or is empty'
+            MASTError(self.__class__.__name__, error)
+            return
+        else:
+            personal_recipe_dict['personal_recipe_list'] = section_content
+
+        for key, value in personal_recipe_dict.items():
+            options.set_item(section_name, key, value)
+
     def parse_ingredients_section(self, section_name, section_content, options):
         """Parses the ingredients section and populates the options.
             Section takes the form of:
@@ -558,10 +596,10 @@ class InputParser(MASTObj):
         for line in section_content:
             line = line.strip()
             #if 'images' in line:
-            #    line = line.split(self.delimiter)
+            #    line = line.split()
             #    images = int(line[1])
             if 'begin' in line:
-                line = line.split(self.delimiter)
+                line = line.split()
                 neblabel = line[1].strip()
                 nebs[neblabel]=dict()
                 nebs[neblabel]['lines']=list()
@@ -676,13 +714,13 @@ class InputParser(MASTObj):
         for line in section_content:
             line = line.strip()
             if 'begin' in line:
-                line = line.split(self.delimiter)
+                line = line.split()
                 label = line[1].strip()
                 phonon[label]=dict()
             elif 'end' in line:
                 pass
             else:
-                line = line.split(self.delimiter, 1)
+                line = line.split()
                 phonon[label][line[0]] = line[1]
 
         options.set_item(section_name, 'phonon', phonon)
@@ -770,7 +808,7 @@ class InputParser(MASTObj):
         """Parses the summary section and populates the options."""
         mast_dict = dict()
         for line in section_content:
-            line = line.split(self.delimiter, 1)
+            line = line.split()
             mast_dict.setdefault(line[0], []).append(line[1].strip())
 
         for key, value in mast_dict.items():
