@@ -2,7 +2,7 @@
 # This code is part of the MAterials Simulation Toolkit (MAST)
 # 
 # Maintainer: Tam Mayeshiba
-# Last updated: 2014-04-25
+# Last updated: 2014-06-13 by Zhewen Song
 ##############################################################
 import os, math
 
@@ -10,9 +10,10 @@ from MAST.utility import MASTObj
 from MAST.utility import InputOptions
 from MAST.utility import MASTError
 from MAST.utility import Metadata
+from MAST.utility import dirutil
 
 ALLOWED_KEYS = {\
-                 'templateFile'    : (str, None, 'template file name'),\
+                 'templateFile'    : (list, None, 'template file name'),\
                  'inputOptions'    : (InputOptions, None, 'input options parsed using input parser'),\
                  'personalRecipe'  : (str, None, 'personalized recipe file'),\
                  'working_directory' : (str, None, 'Working directory'),
@@ -44,24 +45,19 @@ class RecipeTemplateParser(MASTObj):
         """ Parses the template recipe file and creates
             the personalized recipe file
         """
-        if self.template_file is None:
-            raise MASTError(self.__class__.__name__, "Template file not provided!")
-        
-        self.template_file = os.path.join(os.getenv("MAST_RECIPE_PATH"),self.template_file)
-
-
-        if not os.path.exists(self.template_file):
-            raise MASTError(self.__class__.__name__, "Template file not found!")
+        if len(self.template_file) == 0:
+            raise MASTError(self.__class__.__name__, "Recipe contents not provided!")
 
         if self.input_options is None:
             raise MASTError(self.__class__.__name__, "Input Options not provided!")
-
         if self.personal_recipe is None:
             raise MASTError(self.__class__.__name__, "Personal recipe file not provided!")
         
         #fetch required paramaters
-        f_ptr           = open(self.template_file, "r")
-        o_ptr           = open(self.personal_recipe, "w")
+        #f_ptr           = open(self.template_file, "r")
+        recipe_contents = list(self.template_file)
+        #print recipe_contents
+        o_ptr           = open(self.personal_recipe, "a")
         system_name     = self.input_options.get_item("mast", "system_name", "sys")
         n_defects       = self.input_options.get_item("defects", "num_defects", 0)
         d_defects       = self.input_options.get_item("defects","defects")
@@ -73,7 +69,10 @@ class RecipeTemplateParser(MASTObj):
         chunkcount=0
         mychunk=list()
         modchunk=False
-        for line in f_ptr.readlines():
+        for line in recipe_contents: #f_ptr.readlines():
+            duplicate = line
+            if ((len(duplicate) - len(duplicate.lstrip(' '))) % 4 != 0):
+                raise MASTError("parsers/recipetemplateparser", "Recipe at %s contains incorrect number of whitespace chars at the beginning of the line! Please convert all indentations to the appropriate number of groups of four spaces." % line)
             if '\t' in line:
                 raise MASTError("parsers/recipetemplateparser","The tab character exists in recipe template %s. Please convert all tabs to the appropriate number of groups of four spaces." % self.template_file)
             if '{begin}' in line:
@@ -87,14 +86,22 @@ class RecipeTemplateParser(MASTObj):
             self.chunks.append(list(mychunk))
         #for chunk in self.chunks:
         #    print chunk
+
+        input_options_keys = self.input_options.get_sections()
+        key = 'personal_recipe'
         expandedlist=list()
-        for chunk in self.chunks:
-            expanded=self.parse_chunk(chunk)
-            expandedlist.extend(expanded)
-        o_ptr.writelines(expandedlist)
-        f_ptr.close()
-        o_ptr.close()
-        return recipe_name
+        if key in input_options_keys:
+            return expandedlist
+        else:
+            for chunk in self.chunks:
+                expanded=self.parse_chunk(chunk)
+                expandedlist.extend(expanded)
+            o_ptr.write("$personal_recipe\n")
+            o_ptr.writelines(expandedlist)
+            o_ptr.write("$end\n")
+            #f_ptr.close()
+            o_ptr.close()
+            return expandedlist
         #self.chunks[chunkcount]=dict()
         #self.chunks[chunkcount]['modify'] = modchunk
         #    #line             = line.strip()
@@ -117,11 +124,12 @@ class RecipeTemplateParser(MASTObj):
                 expandedchunk <list>: List of lines
         """
         origchunk = list(chunk)
-        expandedchunk=list()
+        expandedchunk = list()
         needsdefects=0
         needscharges=0
         needsphonons=0
         needsnebs=0
+        needsscaling=0
         for line in chunk:
             if "<N>" in line:
                 needsdefects=1
@@ -135,76 +143,92 @@ class RecipeTemplateParser(MASTObj):
                 needsnebs=1
             if "<E>" in line:
                 needsnebs=1
+            if "<S>" in line:
+                needsscaling=1
+        d_scaling       = self.input_options.get_item("structure","scaling")
         d_defects       = self.input_options.get_item("defects","defects")
-        d_nebs         = self.input_options.get_item("neb","nebs")
-        if needsdefects == 1:
-            mydefects=d_defects.keys()
-            mydefects.sort()
-            for defectname in mydefects:
-                for charge in d_defects[defectname]['charge']:
-                    if charge < 0:
-                        mycharge = 'q=n' + str(int(math.fabs(charge)))
-                    else:
-                        mycharge = 'q=p' + str(int(charge))
-                    if needsphonons == 1:
-                        if len(d_defects[defectname]['phonon'].keys()) > 0:
-                            phononkeys = d_defects[defectname]['phonon'].keys()
-                            phononkeys.sort()
-                            for phonon in phononkeys:
-                                for line in origchunk:
-                                    newline = line.replace("<N>", defectname)
-                                    if needscharges == 1:
-                                        newline = newline.replace("<Q>", mycharge)
-                                    newline = newline.replace("<P>", phonon)
-                                    expandedchunk.append(newline)
-                        else:
-                            pass
-                    else:
-                        for line in origchunk:
-                            newline = line.replace("<N>", defectname)
-                            if needscharges == 1:
-                                newline = newline.replace("<Q>", mycharge)
-                            expandedchunk.append(newline)
-        elif needsnebs == 1:
-            nebkeys = d_nebs.keys()
-            nebkeys.sort()
-            for neblabel in nebkeys:
-                defbegin = neblabel.split('-')[0]
-                defend = neblabel.split('-')[1]
-                chargebegin = d_defects[defbegin]['charge']
-                chargeend = d_defects[defend]['charge']
-                chargeboth = set(chargebegin) & set(chargeend)
-                for charge in chargeboth:
-                    if charge < 0:
-                        mycharge = 'q=n' + str(int(math.fabs(charge)))
-                    else:
-                        mycharge = 'q=p' + str(int(charge))
-                    if needsphonons == 1:
-                        if len(d_nebs[neblabel]['phonon'].keys()) > 0:
-                            phononkeys = d_nebs[neblabel]['phonon'].keys()
-                            phononkeys.sort()
-                            for phonon in phononkeys:
-                                for line in origchunk:
-                                    newline = line.replace("<B>", defbegin)
-                                    newline = newline.replace("<E>", defend)
-                                    newline = newline.replace("<B-E>", neblabel)
-                                    if needscharges == 1:
-                                        newline = newline.replace("<Q>", mycharge)
-                                    newline = newline.replace("<P>", phonon)
-                                    expandedchunk.append(newline)
-                        else:
-                            pass
-                    else:
-                        for line in origchunk:
-                            newline = line.replace("<B>", defbegin)
-                            newline = newline.replace("<E>", defend)
-                            newline = newline.replace("<B-E>", neblabel)
-                            if needscharges == 1:
-                                newline = newline.replace("<Q>", mycharge)
-                            expandedchunk.append(newline)
+        d_nebs          = self.input_options.get_item("neb","nebs")
+        
+        if needsscaling == 1:
+            scalingsize = d_scaling.keys()
+            scalingsize.sort()
+        else: scalingsize = ['1x1x1']
 
-        else:
-            expandedchunk = list(origchunk)
+        for size in scalingsize:
+            if needsdefects == 1:
+                mydefects=d_defects.keys()
+                mydefects.sort()
+                for defectname in mydefects:
+                    for charge in d_defects[defectname]['charge']:
+                        if charge < 0:
+                            mycharge = 'q=n' + str(int(math.fabs(charge)))
+                        else:
+                            mycharge = 'q=p' + str(int(charge))
+                        if needsphonons == 1:
+                            if len(d_defects[defectname]['phonon'].keys()) > 0:
+                                phononkeys = d_defects[defectname]['phonon'].keys()
+                                phononkeys.sort()
+                                for phonon in phononkeys:
+                                    for line in origchunk:
+                                        newline = line.replace("<N>", defectname)
+                                        if needscharges == 1:
+                                            newline = newline.replace("<Q>", mycharge)
+                                        if needsscaling == 1:
+                                            newline = newline.replace("<S>",size)
+                                        newline = newline.replace("<P>", phonon)
+                                        expandedchunk.append(newline)
+                        else:
+                            for line in origchunk:
+                                newline = line.replace("<N>", defectname)
+                                if needscharges == 1:
+                                    newline = newline.replace("<Q>", mycharge)
+                                if needsscaling == 1:
+                                    newline = newline.replace("<S>",size)
+                                expandedchunk.append(newline)
+            elif needsnebs == 1:
+                nebkeys = d_nebs.keys()
+                nebkeys.sort()
+                for neblabel in nebkeys:
+                    defbegin = neblabel.split('-')[0]
+                    defend = neblabel.split('-')[1]
+                    chargebegin = d_defects[defbegin]['charge']
+                    chargeend = d_defects[defend]['charge']
+                    chargeboth = set(chargebegin) & set(chargeend)
+                    for charge in chargeboth:
+                        if charge < 0:
+                            mycharge = 'q=n' + str(int(math.fabs(charge)))
+                        else:
+                            mycharge = 'q=p' + str(int(charge))
+                        if needsphonons == 1:
+                            if len(d_nebs[neblabel]['phonon'].keys()) > 0:
+                                phononkeys = d_nebs[neblabel]['phonon'].keys()
+                                phononkeys.sort()
+                                for phonon in phononkeys:
+                                    for line in origchunk:
+                                        newline = line.replace("<B>", defbegin)
+                                        newline = newline.replace("<E>", defend)
+                                        newline = newline.replace("<B-E>", neblabel)
+                                        if needscharges == 1:
+                                            newline = newline.replace("<Q>", mycharge)
+                                        if needsscaling == 1:
+                                            newline = newline.replace("<S>",size)
+                                        newline = newline.replace("<P>", phonon)
+                                        expandedchunk.append(newline)
+                        else:
+                            for line in origchunk:
+                                newline = line.replace("<B>", defbegin)
+                                newline = newline.replace("<E>", defend)
+                                newline = newline.replace("<B-E>", neblabel)
+                                if needscharges == 1:
+                                    newline = newline.replace("<Q>", mycharge)
+                                if needsscaling == 1:
+                                    newline = newline.replace("<S>",size)                            
+                                expandedchunk.append(newline)
+            elif needsscaling==1:
+                for line in origchunk:
+                    newline = line.replace("<S>",size)
+                    expandedchunk.append(newline)
+            else: expandedchunk = list(origchunk)
         return expandedchunk
         #origchunk = list(expandedchunk)
         #expandedchunk=list()
