@@ -5,7 +5,6 @@
 # Maintainer: Tam Mayeshiba
 # Last updated: 2014-04-25
 ##############################################################
-from MAST.controllers.mastmon import MASTmon
-
-mastmon = MASTmon()
+from MAST.controllers.mastmon import MASTMon
+mastmon = MASTMon()
 #mastmon.run(1) #Use 0 for non-verbose
